<?php
/**
* Клас для метода доставки на ПВЗ
*/

if (KC2008_delivery_plugin::isWCactive()) {
    function KC2008_shipping_method_init()
    {
        if (! class_exists('KC2008_Pickup_Method')) {
            class KC2008_Pickup_Method extends WC_Shipping_Method
            {
                /**
                * Constructor KC2008 Delivery Pikup class
                *
                * @access public
                * @return void
                */
                public function __construct($instance_id = 0)
                {
                    $this->instance_id        = absint($instance_id);
                    $this->id                 = 'KC2008_Pickup_Method';
                    $this->method_title       = __('KC2008_Pickup_Method', MEASOFT_PLUGIN_DOMAIN);
                    $this->method_description = $this->get_option('description');
                    $this->enabled            = $this->get_option('enabled');

                    $this->supports           = array(
                    'shipping-zones',
                    'settings',
                    'instance-settings',
                    'instance-settings-modal',
                );

                    $this->init();
                }

                /**
                * Init settings
                *
                * @access public
                * @return void
                */
                public function init()
                {
                    // Загрузка API настроек
                    $this->init_form_fields(); // Это часть настроек API. Переопределите метод, чтобы добавить свои собственные настройки
                    $this->init_settings(); // Это часть настроек API. Загружает настройки, которые вы ранее инициализировали.

                    $this->title = __('KC2008_Pickup_Method', MEASOFT_PLUGIN_DOMAIN); //вывод заголовка при выборе способов доставки
                    $this->title = $this->get_option('title');

                    // Сохранение настроек в админке
                    add_action('woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ));
                }

                /**
                * Настройки метода доставки
                * @return void
                */
                public function init_form_fields()
                {
                    $this->form_fields = array(

                        'enabled' => array(
                            'title' => __('Activate', MEASOFT_PLUGIN_DOMAIN),
                            'type' => 'checkbox',
                            'description' => __('Activate delivery method', MEASOFT_PLUGIN_DOMAIN),
                            'default' => 'yes'
                        ),

                        'title' => array(
                            'title' => __('Title', MEASOFT_PLUGIN_DOMAIN),
                            'type' => 'text',
                            'description' => __('Title delivery method', MEASOFT_PLUGIN_DOMAIN),
                            'default' => __('KC2008_Pickup_Method', MEASOFT_PLUGIN_DOMAIN)
                        ),

                        'description' => array(
                            'title' => __('Description', MEASOFT_PLUGIN_DOMAIN),
                            'type' => 'text',
                            'description' => __('Description delivery method', MEASOFT_PLUGIN_DOMAIN),
                            'default' => __('Choose for using', MEASOFT_PLUGIN_DOMAIN)
                        ),

                    );
                }

                /**
                * Рассчет стоимости доставки
                *
                * @access public
                * @param mixed $package
                * @return void
                */
                public function calculate_shipping($package = array())
                {
                   

                        $options = get_option('KC2008_delivery_option');
                        $measoft = new MeasoftCourier($options['login'], $options['password'], $options['code']);

                        if (isset($options['city'])) {
                            $townfrom = $options['city'];
                        } elseif (get_option('woocommerce_store_city')) {
                            $townfrom = get_option('woocommerce_store_city');
                        } else {
                            $townfrom = false;
                        }

                        if (WC()->customer->get_shipping_city()) {
                            $townto = WC()->customer->get_shipping_city();
                        } elseif (WC()->customer->get_billing_city()) {
                            $townto = WC()->customer->get_billing_city();
                        } else {
                            $townto = 1;
                        }

                        $items = WC()->cart->get_cart();
                        $mass = 0;
						$default_weight = !empty($options['KC2008_weight']) ? $options['KC2008_weight'] : KC2008_delivery_plugin::getDefaultWeight(); 
                        $packages = [];
                        foreach ($items as $item => $values) {
							$product_weight = 0;
                            $_product = wc_get_product($values['data']->get_id());
							if(!empty($_product)){
								$product_weight = (float)$_product->get_weight();
							}
                            if(empty($product_weight)) $product_weight = $default_weight;
                            $mass += $product_weight * (float)$values['quantity'];

                            $packages[]= ['attributes' =>[
									'width' => empty($_product) ? '' : $_product->get_width(),
									'height' => empty($_product) ? '' : $_product->get_height(),
									'length' => empty($_product) ? '' : $_product->get_length(),
									'mass' => (float)$product_weight,
									'quantity' => (float)$values['quantity'],
								]
                            ];
                        }

                     

                        if(empty($mass)) $mass = $default_weight;

                        $order_subtotal = WC()->cart->subtotal;
                        $order_discount_total = WC()->cart->get_cart_discount_total();
                        $order_total = $order_subtotal - $order_discount_total;

                        if ($townfrom && $townto) {
                            try {
                                $req = ['order'=>[
                                    'sender' => ['town' => $townfrom], // +
									'receiver' => ['town' => $townto],          // +                        
                                    'weight' => $mass, // +
                                    'service' => isset($options['service_delivery']) ? $options['service_delivery'] : 1, // +
                                    'pricetype' => 'CUSTOMER', // +
                                    'price' => $order_total, // +
                                    'insprice' => $order_total, // +
                                    'packages' => ['package' => $packages],
                                ]];

                                $measoftpvzCode = WC()->session->get('measoftpvzCode');
                                if (!empty($measoftpvzCode)) {
                                    $req['order']['receiver']['pvz'] = $measoftpvzCode;
									$res = $measoft->calculate($req, true);
                                }else{
									$res = false;
								}
								

                            } catch (\Exception $e) {
                                $res = false;
                                $json_error = $e->getMessage();
                            }
                            $res = (float)$res;
							
							WC()->session->set('measoftpvzPrice', false);
							if(!empty($measoftpvzCode) && is_numeric($res) && $res){
								WC()->session->set('measoftpvzPrice', true);
							}
							
                            /* if ($res) */ {
                                $res = $res * (float)$options['coefficient'] + $options['add_cost'];
								$ks2008_differential_delivery_max = get_option('ks2008_differential_delivery_max');								
								if(!empty($ks2008_differential_delivery_max) && ((float)$ks2008_differential_delivery_max <= (float)$order_total)) $res = 0;	
								
								
								
                                $rate = array(
                                    'id' => $this->id,
                                    'label' => $this->title,
                                    'cost' => (empty($measoftpvzCode)) ? 0 : $res,
                                    'calc_tax' => 'per_item'
                                );

                                // Register the rate
                                $this->add_rate($rate);
                            }
                        }

                    
                }
                /**
                * Проверка доступности метода
                * @param array $package
                * @return bool
                */
                public function is_available($package)
                {
                    return $this->is_enabled();
                }
            }
        }
    }

    //инициализация класса с новым методом доставки
    add_action('woocommerce_shipping_init', 'KC2008_shipping_method_init');

    //добавление нового метода доставки к существующим
    function add_KC2008_shipping_method($methods)
    {
        $methods['KC2008_Pickup_Method'] = 'KC2008_Pickup_Method';
        return $methods;
    }
    add_filter('woocommerce_shipping_methods', 'add_KC2008_shipping_method');
}
