<?php
/**
 * Plugin Name:  MeaSoft Delivery Plugin
 * Description:  Плагин интеграции MeaSoft
 * Plugin URI:   http://courierexe.ru/integration.htm
 * Update URI: 	 https://courierexe.ru/wordpress/
 * Version:      3.32
 * Author: Measoft 
 * Author URI: http://courierexe.ru
 */


if (! defined('ABSPATH')) {
    exit;
} // close from direct request

define('KC2008_PLUGIN_VERSION', '3.31');
define('MEASOFT_PLUGIN_DOMAIN', 'measoft');

if (! class_exists('KC2008_delivery_plugin')) {
    final class KC2008_delivery_plugin
    {
        protected static $_instance = null;

        public $options = null;

        public static function instance()
        {
            if (is_null(self::$_instance)) {
                self::$_instance = new self();
            }
            return self::$_instance;
        }
		
		

        /**
        * Подключение всех необходимых файлов
        * @return void
        */
        public function includes()
        {
            require_once('class/MeasoftCourier.php'); // SDK

            require_once('KC2008_options_page.php'); // main template in admin
            require_once('KC2008_order_form_template.php');
            require_once('KC2008_order_api.php');
            require_once('KC2008_Pickup_Method.php');
            require_once('KC2008_delivery_courier_method.php');

            require_once('MeasoftAjaxController.php');
            require_once('MeasoftPaytypeController.php');
        }

        public function __construct()
        {
            global $wp;
            $this->includes();

            $ajaxController = new MeasoftAjaxController();
            $ajaxController->init();
            
			add_action('woocommerce_init', array($this, 'register_measoft_session'), 99);
            add_action('admin_menu', array($this, 'KC2008_delivery_setup_menu')); // добавление пункта в меню админ-панели
            add_action('admin_init', array($this, 'KC2008_register_settings')); // создание страницы настроек
            add_action('woocommerce_checkout_update_order_meta', array($this, 'ks2008_save_pvz_field')); // сохранение дополнительных полей на странице корзины
            add_action('woocommerce_after_order_notes', array($this, 'ks2008_pvz_checkout_field')); // вывод дополнительных полей на странице корзины
			add_action('woocommerce_after_shipping_rate', array($this, 'measoft_map_display'), 10, 2);
			add_action('init', array($this, 'measoft_load_textdomain'));

            // создание места для вывода формы отправки заказа в АПИ
            add_action('add_meta_boxes_shop_order', function () {
                add_meta_box(
                    'kc2008_api_box',
                    __('Send to Measoft', MEASOFT_PLUGIN_DOMAIN ),
                    array('KC2008_order_form_template', 'KC2008_set_order_form_template'),
                    'shop_order',
                    'side',
                    'default'
                );
            }, 20);

  			add_filter( 'update_plugins_courierexe.ru',  array($this, 'update_plugins_measoft'), 10, 4);	
			add_filter( 'plugins_api',  array($this, 'update_plugins_api'), 10, 3);	
			
			add_filter( 'woocommerce_cart_shipping_method_full_label', array( $this, 'add_free_to_shipping_label_for_zero_cost' ), 10, 2 );
			

            // добавление возможности выполнения крон-задач каждые пол часа
            add_filter('cron_schedules', function ($list) {
                $list['every_30min'] = array(
                    'schedule' => 'half_an_hour',
                    'interval' => 1800,
                    'display' => 'Half an hour',
                );
                return $list;
            });

            register_activation_hook(__FILE__, function () {
                // удалим на всякий случай все такие же задачи cron, чтобы добавить новые с "чистого листа"
                // это может понадобиться, если до этого подключалась такая же задача неправильно (без проверки что она уже есть)
                wp_clear_scheduled_hook('ks2008_hourly_event');

                // добавим новую cron задачу
                wp_schedule_event(time(), 'every_30min', 'ks2008_hourly_event');
            });

            /**
            * Обновление статусов заказа
            * @return bool
            */
            add_action('ks2008_hourly_event', function () {
                global $woocommerce, $wpdb;

                $statuses = array(
                    'NEW' => __('NEW', MEASOFT_PLUGIN_DOMAIN ),
                    'PICKUP' => __('PICKUP', MEASOFT_PLUGIN_DOMAIN ),
                    'ACCEPTED' => __('ACCEPTED', MEASOFT_PLUGIN_DOMAIN ),
                    'INVENTORY' => __('INVENTORY', MEASOFT_PLUGIN_DOMAIN ),
                    'DEPARTURING' => __('DEPARTURING', MEASOFT_PLUGIN_DOMAIN ),
                    'DEPARTURE' => __('DEPARTURE', MEASOFT_PLUGIN_DOMAIN ),
                    'DELIVERY' => __('DELIVERY', MEASOFT_PLUGIN_DOMAIN ),
                    'COURIERDELIVERED' => __('COURIERDELIVERED', MEASOFT_PLUGIN_DOMAIN ),
                    'COMPLETE' => __('COMPLETE', MEASOFT_PLUGIN_DOMAIN ),
                    'PARTIALLY' => __('PARTIALLY', MEASOFT_PLUGIN_DOMAIN ),
                    'COURIERRETURN' => __('COURIERRETURN', MEASOFT_PLUGIN_DOMAIN ),
                    'CANCELED' => __('CANCELED', MEASOFT_PLUGIN_DOMAIN ),
                    'RETURNING' => __('RETURNING', MEASOFT_PLUGIN_DOMAIN ),
                    'RETURNED' => __('RETURNED', MEASOFT_PLUGIN_DOMAIN ),
                    'CONFIRM' => __('CONFIRM', MEASOFT_PLUGIN_DOMAIN ),
                    'DATECHANGE' => __('DATECHANGE', MEASOFT_PLUGIN_DOMAIN ),
                    'NEWPICKUP' => __('NEWPICKUP', MEASOFT_PLUGIN_DOMAIN ),
                    'UNCONFIRM' => __('UNCONFIRM', MEASOFT_PLUGIN_DOMAIN ),
                    'PICKUPREADY' => __('PICKUPREADY', MEASOFT_PLUGIN_DOMAIN ),
                    'AWAITING_SYNC'=>__('AWAITING_SYNC', MEASOFT_PLUGIN_DOMAIN ),
                );

                $options = get_option('KC2008_delivery_option');
                $measoft = new MeasoftCourier($options['login'], $options['password'], $options['code']);

                $neworders = $measoft->changedOrdersRequest();
                // error_log(print_r($neworders, true));
                $commitLastStatus = false;
                if ($neworders && $neworders->attributes()->{'count'} != 0) {
                    if ($neworders->attributes()->{'count'} < 2) {
                        $sql = "SELECT * FROM $wpdb->postmeta WHERE meta_key = 'KC2008_order_number' AND  meta_value = '".$neworders->order['orderno']."' LIMIT 1";
                        $posts = $wpdb->get_results($sql, ARRAY_A);
                        if ($posts) {
                            $order = wc_get_order($posts[0]['post_id']);
                            $neworders = json_decode(json_encode($neworders));
                            if ($order) {
                                if (isset($statuses[(string)$neworders->order->status])) {
                                    $status = $statuses[(string)$neworders->order->status];
                                } else {
                                    $status = (string)$neworders->order->status;
                                }

                                $order->update_meta_data('KC2008_order_status', (string)$status);
								$order->save_meta_data();
                                $note = __('New order status is', MEASOFT_PLUGIN_DOMAIN ).' '.$status;
                                $order->add_order_note($note);
                                $commitLastStatus = true;

                                if ($options['statuses']) {
                                    $status_option = $options['status_'.(string)$neworders->order->status];
                                    if ($status_option) {
                                        $order->update_status($status_option);
                                    }
                                }
                                $order->save();
                            }
                        }
                    } else {
                        foreach ($neworders->order as $orders) {
                            $sql = "SELECT * FROM $wpdb->postmeta WHERE meta_key = 'KC2008_order_number' AND  meta_value = '".$orders->attributes()->{'orderno'}."' LIMIT 1";
                            $posts = $wpdb->get_results($sql, ARRAY_A);
                            if ($posts) {
                                $order = wc_get_order($posts[0]['post_id']);
                                $orders = json_decode(json_encode($orders));
                                if ($order) {
                                    if (isset($statuses[(string)$orders->status])) {
                                        $status = $statuses[(string)$orders->status];
                                    } else {
                                        $status = (string)$orders->status;
                                    }
                                    $order->update_meta_data('KC2008_order_status', $status);
									$order->save_meta_data();
                                    $note = __('New order status is', MEASOFT_PLUGIN_DOMAIN ).' '.$status;
                                    $order->add_order_note($note);
                                    $commitLastStatus = true;

                                    if ($options['statuses']) {
                                        $status_option = $options['status_'.$orders->status];
                                        if ($status_option) {
                                            $order->update_status($status_option);
                                        }
                                    }
                                    $order->save();
                                }
                            }
                        }
                    }
                }

                if ($commitLastStatus) {
                    try {
                        $measoft->commitLastStatusRequest();
                    } catch (\Exception $e) {
                    }
                }
            });

            // При деактивации плагина, обязательно нужно удалить задачу:
            register_deactivation_hook(__FILE__, function () {
                wp_clear_scheduled_hook('ks2008_hourly_event');
            });
        }
		
		/* Init textdomain.
		*/
		public function measoft_load_textdomain() {
			load_plugin_textdomain( 'measoft', false, basename( dirname( __FILE__ ) ) . '/languages/' );
		}
		
		/* Start session
		*/
		function register_measoft_session(){			
			if (is_user_logged_in() || is_admin()) {
				return;
			}
			if (isset(WC()->session)) {
				if (!WC()->session->has_session()) {
					WC()->session->set_customer_session_cookie(true);
				}
			}			
		}
		
		public static function getDataForMeasoft($order, $measoft, $options, $order_id){
			
			$order_data = $order->get_data();
            $measoftPaytypeController = new MeasoftPaytypeController();
			
            $pay_type = !is_null($measoftPaytypeController->getTypeFromGatewayId($order->get_payment_method())) ? $measoftPaytypeController->getTypeFromGatewayId($order->get_payment_method()) : MeasoftCourier::PAYMENT_TYPE_NONE;
			

            $order_shipping_city = $order_data['shipping']['city'];
            $order_billing_city = $order_data['billing']['city'];
			$order_total = $order_data['total'];
		
		

            $city = $order_shipping_city ? $order_shipping_city : $order_billing_city;

            if (get_post_meta($order_id, 'pvzname', true)) {
                $pvzname = get_post_meta($order_id, 'pvzname', true);
            } else {
                $pvzname = '';
            }

            if (get_post_meta($order_id, 'pvzcode', true)) {
                $pvzcode = get_post_meta($order_id, 'pvzcode', true);
            } else {
                $pvzcode = '';
            }

            if (!empty($order_shipping_city)) {
                $pvzcity = $order_shipping_city;
            } else {
                $pvzcity = '';
            }

           
			$order_shipping_address_1 = $order_data['shipping']['address_1'];
            $order_shipping_address_2 = $order_data['shipping']['address_2'];
            $order_billing_address_1 = $order_data['billing']['address_1'];
            $order_billing_address_2 = $order_data['billing']['address_2'];

            if (!empty($pvzname)) {
                $address = $pvzname;
            } else {
                $address = $order_shipping_address_1 ? $order_shipping_address_1 . ' ' . $order_shipping_address_2 : $order_billing_address_1 . ' ' . $order_billing_address_2;
            }
			
			 $shipping_address = $address;
			
            // get weight -->
            $total_weight = 0;
			$default_weight = !empty($options['KC2008_weight']) ? $options['KC2008_weight'] : self::getDefaultWeight();
			$packages = [];			
            foreach ($order->get_items() as $item_key => $item):
				$product_weight = 0;
				$product        = $item->get_product();
				$quantity       = $item->get_quantity();	
                if (!empty($product) && isset($options['articles'])) {               
                    $product_sku    = $product->get_sku();
					try {
						$product_data = $measoft->productGetByArticle($product_sku);
						$product_weight  = $product_data->item->weight;
					}catch(\Exception $e){
						$product_weight  = $default_weight;
					}			

                } else {

                    $product_weight  = empty($product) ? 0 : (float)$product->get_weight();
                }
			
			

			if(empty($product_weight)) $product_weight = $default_weight;
            $total_weight += (float)$product_weight * (float)$quantity;
			
			$packages[]= ['attributes' =>[
									'width' => empty($product) ? '' : $product->get_width(),
									'height' => empty($product) ? '' : $product->get_height(),
									'length' => empty($product) ? '' : $product->get_length(),
									'mass' => (float)$product_weight,
									'quantity' => (float)$item['quantity'],
								]
                            ];

            endforeach;

            if(empty($total_weight)) $total_weight = $default_weight;
            // <-- get weight


            if (isset($options['prefix'])) {
                $orderno = $options['prefix'] . $order->get_id();
            } else {
                $orderno = $order->get_id();
            }

            if (isset($options['city'])) {
                $townfrom = $options['city'];
            } elseif (get_option('woocommerce_store_city')) {
                $townfrom = get_option('woocommerce_store_city');
            } else {
                $townfrom = '';
            }	
			
			$result = [
			'delivery_date' => date('Y-m-d'),
			'delivery_time_from' => '10:00',
			'delivery_time_to' => '18:00',
			'pvz_name' => $pvzname,
			'pvz_adress' => $pvzname,
			'pvz_id' => $pvzcode,
			'pvz' => $pvzcode,
			'pvz_city' => $pvzcity, 			
			'pay_type' => $pay_type,
			'orderno' => $orderno,
			'deliveryprice' => (float)$order->get_total_shipping(),
			'enclosure' => '',
			'instruction' => '',
			'order_id' => $order_id,
			'townfrom' => $townfrom,
			'townto' => $city,
			'weight' => $total_weight,
			'shipping_address' => $shipping_address,
			'post_id' => $order_id	,		
			'order_total' => $order_total,			
			'packages' => $packages,
			];
			
			return $result;

			
		}
		
		
		public static function getWeightDelimiter(){
			return (get_option( 'woocommerce_weight_unit', 'kg' ) == 'kg') ? 1 : 1000;			
		}
		
		public static function getDefaultWeight(){
			return 0.1 * (self::getWeightDelimiter());
		}
		
		
		public function add_free_to_shipping_label_for_zero_cost( $label, $method ) {
			$measoftpvzPriceFound = WC()->session->get('measoftpvzPrice');			
			if (is_numeric($method->cost) && $method->cost <= 0  && ($method->method_id == 'KC2008_shipping_courier_method' || (!empty($measoftpvzPriceFound) && $method->method_id == 'KC2008_Pickup_Method')) ) {
				$label .= ' <strong>('.__("Free", MEASOFT_PLUGIN_DOMAIN ).')</strong>';
			}
			return $label;
		}

        public function KC2008_delivery_setup_menu()
        {
            add_menu_page('Delivery plugin', __( 'KC2008 Settings Page', MEASOFT_PLUGIN_DOMAIN ), 'manage_options', 'delivery-plugin', array('KC2008_options_page', 'KC2008_delivery_set_template'), 'dashicons-admin-site-alt2', 57);
        }

        public function KC2008_register_settings()
        {
            register_setting('KC2008_delivery_options_group', 'KC2008_delivery_option', 'KC2008_callback');
        }

        public function ks2008_save_pvz_field($order_id)
        {
            if (!empty($_POST['pvzcode']) && !empty($_POST['pvzname'])) {
                update_post_meta($order_id, 'pvzcode', $_POST['pvzcode']);
                update_post_meta($order_id, 'pvzname', $_POST['pvzname']);
            }
        }



        public function ks2008_pvz_checkout_field($checkout)
        {
            echo '<div id="ks2008_pvz_checkout" style="display:none;">';
            echo "<div id='ks2008_pvz_map_block'></div>";
			
			$measoftpvzCode = WC()->session->get( 'measoftpvzCode');
			$measoftpvzAddress = WC()->session->get( 'measoftpvzAddress');
			
            woocommerce_form_field('pvzcode', array(
                'type'          => 'text',
                'class'         => array('ks2008_pvzcode form-row-wide'),
                'label'         => '',
                'placeholder'   => '',
            ), $measoftpvzCode);
            woocommerce_form_field('pvzname', array(
                'type'          => 'text',
                'class'         => array('ks2008_pvzname form-row-wide'),
                'label'         => '',
                'placeholder'   => '',
            ), $measoftpvzAddress);

            echo '</div>';
        }


        public function measoft_map_display($shippingMethodCurrent)
        {
            if (is_checkout() && $this->isTariffTypeFromStore($shippingMethodCurrent)) {
				$pvzAddress = WC()->session->get( 'measoftpvzAddress');
                include 'map-button.php';
            }

        }

        protected function getData($url)
        {
            $res = wp_remote_get($url);
            return $res['response']['message'] == 'OK' ? $res['body'] : false;
        }

        protected function getChangelog()
        {
            $url = 'https://courierexe.ru/wordpress/changelog.txt';
            $res = $this->getData($url);
            return $res ? $res : __('Minor updates', MEASOFT_PLUGIN_DOMAIN );


        }

        protected function getVersion()
        {
            $url = 'https://courierexe.ru/wordpress/version.txt';
            $res = $this->getData($url);        
            return $res ? $res : KC2008_PLUGIN_VERSION;

        }


        public function update_plugins_api($result, $action, $args)
        {		
			if($args->slug == 'measoft'){
				$changelog = $this->getChangelog();			
				$result = (object)[
                'name' => __('KC2008 Integration Plugin', MEASOFT_PLUGIN_DOMAIN ),
                'slug' => MEASOFT_PLUGIN_DOMAIN,
                'sections' => [
                    'description' => __('KC2008 is Integration System', MEASOFT_PLUGIN_DOMAIN ),
                    'changelog' => $changelog ? $changelog : __('Minor updates', MEASOFT_PLUGIN_DOMAIN ),
					]
				];
			}            
            return $result;
        }

        public function update_plugins_measoft($update, $plugin_data, $plugin_file, $locales)
        {
            $version = $this->getVersion();


            $data = [
                'slug' => MEASOFT_PLUGIN_DOMAIN,
                'icons' => ['default' => 'https://courierexe.ru/wordpress/logo.svg'],
                'version' => $version,
                'tested' => '6.2.2',
                'package' => 'https://courierexe.ru/wordpress/wordpress.php',
                'url' => 'https://courierexe.ru/wordpress',
            ];
            return $data;
        }
		


        public function isTariffTypeFromStore($shippingMethodCurrent)
        {
            if ($shippingMethodCurrent->get_method_id() !== 'KC2008_Pickup_Method') {
                return false;
            }

            $shippingMethodIdSelected = WC()->session->get('chosen_shipping_methods')[0];

            if ($shippingMethodCurrent->get_id() !== $shippingMethodIdSelected) {
                return false;
            }

            return true;
        }
		public static function isWCactive(){
			
			$active = false; 
			
			if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
					require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
			}
       
			if ( is_multisite() && is_plugin_active_for_network( plugin_basename(__FILE__) ) ) {
				
						$active = is_plugin_active_for_network('woocommerce/woocommerce.php') ? true : false; 					
			} else {
				
						$active =  is_plugin_active( 'woocommerce/woocommerce.php') ? true : false;    
			}
		
			return $active;
		}


    }
}


if (! function_exists('callTheClass')) {
    function callTheClass()
    {
        return KC2008_delivery_plugin::instance();
    }
}
callTheClass();
