<?php
/**
 * Шаблон формы отправки заказа в АПИ
 */

if (! defined('ABSPATH')) {
    exit;
}

if (! class_exists('KC2008_order_form_template')) {
    class KC2008_order_form_template
    {
        protected static $_instance = null;

        public static function instance()
        {
            if (is_null(self::$_instance)) {
                self::$_instance = new self();
            }
            return self::$_instance;
        }

        public function __construct()
        {
            add_action('admin_enqueue_scripts', array($this, 'add_admin_iris_scripts'));
        }

        public function add_admin_iris_scripts($hook)
        { // js file and set myajax.url
            wp_localize_script(
                'jquery',
                'myajax',
                array(
                    'url' => admin_url('admin-ajax.php'),
                    'nonce' => wp_create_nonce('myajax-nonce')
                )
            );
			$options = get_option('KC2008_delivery_option');
            wp_localize_script(
                'jquery',
                'ks2008client',
                array(
                    'id' => is_array($options) ? $options['code'] : '',
					'code' => is_array($options) ? $options['client_code'] : '',
                    'map_width' => is_array($options) ? $options['map_width'] : '',
                    'map_height' => is_array($options) ? $options['map_height'] : '',					
					'saving' => __('Saving...', MEASOFT_PLUGIN_DOMAIN),
					'choosepickuppoint' => __('Choose pickup point', MEASOFT_PLUGIN_DOMAIN),
					'field3snotempty' => __('The fileds Login, Password, Service Client must not be empty', MEASOFT_PLUGIN_DOMAIN),
					'checking' => __('Checking...', MEASOFT_PLUGIN_DOMAIN),
					'success' => __('Success', MEASOFT_PLUGIN_DOMAIN),
					'error' => __('Error', MEASOFT_PLUGIN_DOMAIN),
					'error_town' => __('Error: Delivery town not set', MEASOFT_PLUGIN_DOMAIN),
					'error_weight' => __('Error: Delivery weight not set', MEASOFT_PLUGIN_DOMAIN),
					'error_town_sender' => __('Error: Sender town not set', MEASOFT_PLUGIN_DOMAIN),
					'calculating' => __('Calculating', MEASOFT_PLUGIN_DOMAIN),
					'error_weekend' => __('We can\'t create order on weekend', MEASOFT_PLUGIN_DOMAIN),
					'error_price' => __('Delivery price not set', MEASOFT_PLUGIN_DOMAIN),
					'error_delivery_date' => __('Delivery date not set', MEASOFT_PLUGIN_DOMAIN),
					'error_delivery_time_from' => __('Delivery time to not set', MEASOFT_PLUGIN_DOMAIN),
					'error_delivery_time_to' => __('Delivery time from not set', MEASOFT_PLUGIN_DOMAIN),
					'error_number_order' => __('Order number date not set', MEASOFT_PLUGIN_DOMAIN),
					'sending' => __('Sending...', MEASOFT_PLUGIN_DOMAIN),
					'order_n' => __('Order #', MEASOFT_PLUGIN_DOMAIN),
					'order_s' => __('has send', MEASOFT_PLUGIN_DOMAIN),
                )
            );

            wp_enqueue_script('KC2008-measoft-map-script', 'https://home.courierexe.ru/js/measoft_map.js', array('jquery'), KC2008_PLUGIN_VERSION, 1);
            wp_enqueue_script('KC2008-order-form-script', plugins_url('js/order-form-script.js', __FILE__), array('jquery'), KC2008_PLUGIN_VERSION, 1);
            wp_enqueue_style('KC2008-order-form-style', plugins_url('css/order-form-style.css', __FILE__), array(), KC2008_PLUGIN_VERSION);

            if (! wp_script_is('jquery-ui-autocomplete', 'enqueued')) {
                wp_enqueue_script('jquery-ui-autocomplete');
            }
        }

        public static function KC2008_set_order_form_template()
        {
            global $post;
            $order = wc_get_order($post->ID);            
            $options = get_option('KC2008_delivery_option');
			if(!$options || !is_array($options)){
				$options = [
					'login' => '',
					'password' => '',
					'code' => '',
				];
			}
            $measoft = new MeasoftCourier($options['login'], $options['password'], $options['code']); // test query with php
			$data = KC2008_delivery_plugin::getDataForMeasoft($order, $measoft, $options, $post->ID);
			
			
            if ($order->get_meta('KC2008_order_number')) {
                echo "<p>".__('Order number #', MEASOFT_PLUGIN_DOMAIN)." ".$order->get_meta('KC2008_order_number')." ".__('has send', MEASOFT_PLUGIN_DOMAIN)."</p>";
                echo "<p>".__('Status', MEASOFT_PLUGIN_DOMAIN).": ".$order->get_meta('KC2008_order_status')."</p>";
            } else {
                ?>
                <div class="form-table ks2008_order_delivery_table">

                    <p>
                        <label for="delivery_date"><?php print __('Delivery date', MEASOFT_PLUGIN_DOMAIN);?> <span>*</span></label>
                        <input required class="delivery_date" name="delivery_date" id="delivery_date" type="date" value="<?php echo $data['delivery_date']; ?>" min="<?php echo $data['delivery_date']; ?>">
                    </p>
                    <p>
                        <hr>
                        <label for="delivery_time_from"><strong><?php print __('Delivery time', MEASOFT_PLUGIN_DOMAIN);?></strong></label>
                        <br>
                        <span><?php print __('From', MEASOFT_PLUGIN_DOMAIN);?>:</span>
                        <input class="delivery_time" name="delivery_time_from" id="delivery_time_from" type="time" value="<?php echo $data['delivery_time_from']; ?>">
                        <br>
                        <br>
                        <span><?php print __('To', MEASOFT_PLUGIN_DOMAIN);?>:</span>
                        <input class="delivery_time" name="delivery_time_to" id="delivery_time_to" type="time" value="<?php echo $data['delivery_time_to']; ?>">
                    </p>

                    <!-- <p> -->
                        <hr>
                        <label><strong><?php print __('Choose Pickup Point', MEASOFT_PLUGIN_DOMAIN);?>:</strong></label><br>

                        <br>
                        <div id="measoftMapBlock">

                        </div>
                        <br>
                        <input type="text" autocomplete="off" id="pvzname" name="pvz_name" value="<?php echo htmlspecialchars($data['pvz_name']); ?>" placeholder="<?php print __('Point title', MEASOFT_PLUGIN_DOMAIN);?>" readonly />
                        <br>
                        <input type="hidden" autocomplete="off" class="pvzcode" name="pvz_id" value="<?php echo $data['pvz_id']; ?>" placeholder="Код ПВЗ" readonly />
                        
                        <input id="pvzcity" type="hidden" autocomplete="off" class="pvzcity" name="pvz_city" value="<?php echo htmlspecialchars($data['pvz_city']); ?>" readonly />
                        <br>
                        <button type="button" name="ks2008_clean_pvz" id="ks2008_clean_pvz" class="button button-secondary"><?php print __('Remove pickup point', MEASOFT_PLUGIN_DOMAIN);?></button>

                    <!-- </p> -->

                    <p>
                        <input type="hidden" name="pay_type" value="<?php echo $data['pay_type']; ?>" />
                    </p>

                    <p>
                        <label for="orderno"><?php print __('Order number', MEASOFT_PLUGIN_DOMAIN);?></label>
                        <input class="orderno" name="orderno" id="orderno" type="text" value="<?php echo $data['orderno']; ?>" required />
                    </p>

                    <p>
                        <label for="deliveryprice"><?php print __('Delivery price', MEASOFT_PLUGIN_DOMAIN);?></label>
                        <input class="deliveryprice" name="deliveryprice" id="deliveryprice" type="text" readonly value="<?php echo $data['deliveryprice']; ?>"><br>
                        <button type="button" name="deliveryprice_calculate" id="ks2008_deliveryprice_calculate" class="button button-secondary" style="display:block;"><?php print __('Calculate delivery price', MEASOFT_PLUGIN_DOMAIN);?></button>
                        <span style="display:block;" class="ks2008_deliveryprice_responce"></span>
                    </p>

                    <p>
                        <label for="enclosure"><?php print __('Enclosure', MEASOFT_PLUGIN_DOMAIN);?></label>
                        <textarea id="enclosure" name="enclosure" rows="3" placeholder="<?php print __('Enclosure text', MEASOFT_PLUGIN_DOMAIN);?>"></textarea>
                    </p>

                    <p>
                        <label for="instruction"><?php print __('Instruction', MEASOFT_PLUGIN_DOMAIN);?></label>
                        <textarea id="instruction" name="instruction" rows="3" placeholder="<?php print __('Instruction text', MEASOFT_PLUGIN_DOMAIN);?>"></textarea>
                    </p>

                    <p style="display:block;">
					   <input type="hidden" name="order_total" value="<?php echo $data['order_total']; ?>">
                        <input type="hidden" name="order_id" value="<?php echo $data['order_id']; ?>">
                        <input type="hidden" name="townfrom" id="townfrom" value="<?php echo htmlspecialchars($data['townfrom']); ?>">
                        <input type="hidden" name="townto" id="city" value="<?php echo htmlspecialchars($data['townto']); ?>">
                        <input type="hidden" name="weight" id="weight" value="<?php echo $data['weight']; ?>">
                        <input type="hidden" name="shipping_address" id="shipping_address" value="<?php echo htmlspecialchars($data['shipping_address']); ?>">
						<input type="hidden" name="paytype" id="paytype" value="<?php echo htmlspecialchars($data['pay_type']); ?>">
						<input type="hidden" name="packages" id="packages" value="<?php echo htmlspecialchars(json_encode($data['packages'])); ?>">
                    </p>

                    <p>
                        <div class="createOrderBtn button button-primary"><?php print __('Send order', MEASOFT_PLUGIN_DOMAIN);?></div>
                    </p>

                    <p>
                        <div class="alertBlock"></div>
                    </p>
                </div>
            <?php
            }
        }
    }
}

if (! function_exists('callTheOrderTemplateClass')) {
    function callTheOrderTemplateClass()
    {
        return KC2008_order_form_template::instance();
    }
}
callTheOrderTemplateClass();
