/*
Модуль отображения карты пунктов самовывоза
Версия: 3.0

Использование:
1. Прописать в тэге head:
	<script src="https://home.courierexe.ru/js/measoft_map.js" type="text/javascript"></script>

2. Вставить на страницу следующий код:
	<div id="measoftMapBlock">
		<script defer type="text/javascript">
		var measoftObject = measoftMap.config({
			'mapBlock': 'measoftMapBlock',
			'client_id': '8',					// Сюда нужно указать код extra курьерской службы
			'client_code': '1513',				// Сюда можно указать код клиента курьерской службы (необязательный параметр)
			'mapSize': {						// Размер карты
				'width': '650',
				'height': '650'
			},
			'centerCoords': ['55.755814', '37.617635'],
			'showMapButton': '1',
			'showMapButtonCaption': 'Выбрать пункт самовывоза',
			'filter': {
				'acceptcard': 'YES',    // Можно добавлять acceptcash (принимают наличные), acceptcard (Принимают карты), acceptfitting (Есть примерка), acceptindividuals (Если вы-физ. лицо), maxweight - максимальный вес (в формате maxweight: 10.5, для максимального веса в 10.5 кг)
			},
			'allowedFilterParams': ['acceptcash', 'acceptcard', 'acceptfitting', 'store'], // store - Выводить список компаний-филиалов для фильтрации
		}).init();
		</script>
	</div>

	Обязательные параметры:

		mapBlock - атрибут id блока, в котором будет выводиться карта
		client_id - id в системе Курьерская служба		

	Необязательные параметры:

		buttonBlock - атрибут id блока-кнопки
        townBlock - атрибут, содержащий город, если это необходимо
		townRegexp - регулярное выражение, чтобы удалить часть текста из названия города по шаблону, если это необходимо, н-р /\,.+/gsi - чтобы удалить все после запятой
		client_code - код клиента курьерской службы

		mapSize - параметры размера карты:
			width - ширина (по умолчанию 850 пикселей),
			height - высота (по умолчанию 400 пикселей)

		centerCoords - координаты, по которым центрируется карта

		showMapButton - показывать кнопку для вызова карты
			0 - карта выводится в указанном блоке и занимает размер, указанный в параметре mapSize
			1 - карта выводится при нажатии на кнопку в виде окна размером, указанным в параметре mapSize, и сворачивается при повторном нажатии
			2 - кнопка не выводится

		windowFixedPosition - вывод окна карты по центру страницы с затемнением фона. Сбрасывает значение параметра mapSize, открывающееся окно занимает всю страницу.

		showMapButtonCaption - название кнопки вызова карты

		filter - фильтрация пунктов самовывоза по параметрам:
			acceptcash - прием наличных
			acceptcard - возможность оплаты карты
			acceptfitting - возможность примерки
			acceptindividuals - доступность физического лица
			maxweight - максимальный вес (в формате maxweight: 10.5, для максимального веса в 10.5 кг)

		allowedFilterParams - параметры фильтрации, которые доступны для изменения

		choicePvzCallback - callback-функция, на событие выбора ПВЗ


3. Код и адрес выбранного ПВЗ возвращает метод measoftMap.getSelectedPvzData();

*/

;(function (window, undefined) {
    'use strict';

    var myselfLayer = null;
    var marksData = 0;
    var jsLoaded = false;
    var mapReady = false;
    var isOpened = false;
    var town = false;

    var head = document.getElementsByTagName('head')[0];

    var css = [
        'https://unpkg.com/leaflet@1.6.0/dist/leaflet.css',
        'https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.4.1/MarkerCluster.css',
        'https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.4.1/MarkerCluster.Default.css'
    ];

    css.forEach(function (item) {
        var link = document.createElement('link');
        link.rel = 'stylesheet';
        link.type = 'text/css';
        link.href = item;
        head.appendChild(link);
    });

    var leaflet = 'https://unpkg.com/leaflet@1.6.0/dist/leaflet.js';
    var markercluster = 'https://cdnjs.cloudflare.com/ajax/libs/leaflet.markercluster/1.4.1/leaflet.markercluster.js';

    var script = document.createElement('script');
    script.type = 'text/javascript';
    script.src = leaflet;
    head.appendChild(script);
    head.onload = loadMarkercluster();

    // реализуем из-за особенностей инициализации leaflet
    function loadMarkercluster() {
        if (!window.L) {
            window.setTimeout(loadMarkercluster, 100);
            return;
        }
        var script = document.createElement('script');
        script.type = 'text/javascript';
        script.src = markercluster;
        head.appendChild(script);
        jsLoaded = true;
    }

    var settings = {
        'pvzCodeSelector': '',
        'pvzNameSelector': '',
        'pvzAddressSelector': '',
        'mapSearchZoom': '14',
        'mapZoom': '10',
        'showSearchAddress': '1',
        'minSearchCityLength': '1',
        'minSearchStreetLength': '1',
        'showMapButton': '0',
        'clearButton': '0',
        'windowFixedPosition': '0',
        'showMapButtonCaption': 'Выбрать ПВЗ',
        'mapBlock': '',
        'buttonBlock': 'open_map_button',
        'townBlock': '',
        'filter': {
            'acceptcash': '',
            'acceptcard': '',
            'acceptfitting': '',
            'acceptindividuals': '',
            'maxweight': 0

        },
        'allowedFilterParams': {},
        'client_id': '8',
        'client_code': '',
        'centerCoords': ['57.411880', '56.677315'],
        'mapSize': {
            'width': '450',
            'height': '400'
        },
        'choicePvzCallback': '',
		'townRegexp': ''
    };

    var measoftMap = {};
    var map = null;
    measoftMap.objects = [];
    var api_url = 'https://home.courierexe.ru/api/';

    var filterCaptions = {
        'acceptcash': 'Прием наличных',
        'acceptcard': 'Оплата картой',
        'acceptfitting': 'Возможность примерки',
        'store': 'Компания',
    };

    var filterTypes = {
        'acceptcash': 'checked',
        'acceptcard': 'checked',
        'acceptfitting': 'checked',
        'store': 'select',
    };

    var request = new XMLHttpRequest();

    var placeCoords = 0;

    //Функция проверки типа
    function is(obj, type) {
        var c = Object.prototype.toString.call(obj).slice(8, -1);
        return obj !== undefined && obj !== null && c.toLowerCase() === type.toLowerCase();
    }

    //Функция запроса
    function ajax(url, params) {
        request.abort();

        request.open('POST', url, true);
        request.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');

        request.onreadystatechange = function () {
            if (this.readyState === 4) {
                if (this.status >= 200 && this.status < 400) {
                    params['success'](this.responseText);
                } else {
                    // Error :(
                }
            }
        };

        request.send(params['data']);
    }

    measoftMap.config = function (params) {
        //Проверим наличие аргументов
        if (!arguments.length) {
            throw new TypeError('Параметры не найдены');
        }
        //Является ли первый аргумент объектом
        if (is(params, 'Object')) {
            for (var param in params) {
                if (params.hasOwnProperty(param) && settings.hasOwnProperty(param)) {
                    settings[param] = params[param];
                }
            }
            //В ином случае выбросим ошибку
        } else {
            throw new TypeError('Ошибка параметров');
        }

        return measoftMap;
    };

    measoftMap.loadStore = function () {
		let fndSelect  = false;
        if (settings.allowedFilterParams.length > 0) {

            for (let i = 0; i < settings.allowedFilterParams.length; i++) {				
                if (filterCaptions[settings.allowedFilterParams[i]] !== undefined && filterTypes[settings.allowedFilterParams[i]] !== undefined && filterTypes[settings.allowedFilterParams[i]] == 'select') {
					fndSelect=true;
					}
			}
				if(fndSelect){
                    var xml = '<?xml version="1.0" encoding="UTF-8" ?><storelist><auth extra="' + settings.client_id + '"></auth>';
                    if (settings.client_code) {
                        xml += '<client_code>' + settings.client_code + '</client_code>';
                    }
                    xml += '<json>YES</json></storelist>';

                    ajax(api_url, {
                        data: xml,
                        success: function (response) {
                            measoftMap.showMap();
                            if (response !== undefined && response != '') {
                                try {
                                    var store_list = JSON.parse(response);
                                    var selStoreElement = document.getElementById('cb_store');
                                    store_list.forEach(function (store_item) {
                                        if (store_item.disabled != 1) {
                                            let opt = document.createElement('option');
                                            opt.appendChild(document.createTextNode(store_item.name));
                                            opt.value = store_item.code;
                                            selStoreElement.appendChild(opt);
                                        }
                                    });
                                } catch (e) {
                                }
                            }
                        },
                        error: function (jqxhr, status, errorMsg) {
                            console.log(status + ": " + errorMsg);
                        }
                    });
                }else{
					measoftMap.showMap();
				}           
        } else {
            measoftMap.showMap();
        }
    };


    measoftMap.init = function () {
        if (!jsLoaded || !window.L) {
            window.setTimeout(measoftMap.init, 100);
            return;
        }

        document.getElementById(settings.mapBlock).innerHTML = getMapWindowHtml();
        measoftMap.loadStore();
        validateSize();


        measoftMap.preloadImages(['https://home.courierexe.ru/images/map/marker-icon-2x-green.png', 'https://home.courierexe.ru/images/map/marker-shadow.png', 'https://home.courierexe.ru/images/ajax-loader.gif']);
        measoftMap.autocomplete(document.getElementById("searchCity"), 'city');
        measoftMap.autocomplete(document.getElementById("searchStreet"), 'street');

        document.getElementById("searchHouse").addEventListener("keydown", function (e) {
            if (e.keyCode === 13) {
                measoftMap.MoveToAddress();
                e.preventDefault();
            }
        });
        document.getElementById("searchHouse").addEventListener("input", function (e) {
            return false;
        });

    };
	
	


    measoftMap.autocomplete = function (inp, sel_type) {


        /*the autocomplete function takes two arguments,
        the text field element and an array of possible autocompleted values:*/
        var currentFocus;
        /*execute a function when someone writes in the text field:*/
        inp.addEventListener("input", function (e) {

            /// inp.classList.add("loading");


            let val = this.value, xml = '',
                result_list = [],
                selectedCityCode, city_title,
                city_code, region, street_title, coords, coords_lat, coords_lon;
            /*close any already o,pen lists of autocompleted values*/
            closeAllLists();
            if (!val) {
                return false;
            }


            if (val.length < settings.minSearchCityLength) {

                return false;
            }

            if (sel_type == 'city') {
                xml = '<?xml version="1.0" encoding="UTF-8" ?><townlist><conditions><namecontains>' + val + '</namecontains></conditions><limit><limitcount>5</limitcount></limit></townlist>';

            } else {
                let selectedCityCode = document.getElementById('searchCity').getAttribute("data-selectedCityCode");
                xml = '<?xml version="1.0" encoding="UTF-8" ?><streetlist><conditions><town>' + selectedCityCode + '</town><namecontains>' + val + '</namecontains></conditions><limit><limitcount>5</limitcount></limit></streetlist>';

            }

            ajax(api_url, {
                data: xml,
                success: function (response) {

                    if (window.DOMParser) {
                        var parser = new DOMParser();
                        var xmlDoc = parser.parseFromString(response, "text/xml");
                    } else // Internet Explorer
                    {
                        var xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
                        xmlDoc.async = false;
                        xmlDoc.loadXML(response);
                    }


                    if (sel_type == 'city') {
                        result_list = xmlDoc.getElementsByTagName('town');
                    } else {
                        result_list = xmlDoc.getElementsByTagName('street');
                    }


                    currentFocus = -1;
                    /*create a DIV element that will contain the items (values):*/
                    let a = document.createElement("DIV");
                    a.setAttribute("id", inp.getAttribute('id') + "autocomplete-list");
                    a.setAttribute("class", "autocomplete-items");
                    /*append the DIV element as a child of the autocomplete container:*/

                    document.getElementById('map_filter_address_block').appendChild(a);

                    if (sel_type == 'city') {
                        let offsetLeft = document.getElementById("searchCity").offsetLeft;
                        document.getElementById("searchCityautocomplete-list").style.left = offsetLeft + 'px';
                    } else {
                        let offsetLeft = document.getElementById("searchStreet").offsetLeft;
                        document.getElementById("searchStreetautocomplete-list").style.left = offsetLeft + 'px';
                    }


                    /*for each item in the array...*/

                    if (result_list.length > 0) {

                        for (let i = 0; i < result_list.length; i++) {

                            if (sel_type == 'city') {
                                city_title = result_list[i].getElementsByTagName('name')[1].textContent + '';
                                city_code = result_list[i].getElementsByTagName('code')[0].textContent + '';
                                region = result_list[i].getElementsByTagName('city')[0].getElementsByTagName('name')[0].textContent + '';
                                coords = result_list[i].getElementsByTagName('coords')[0];
                                coords_lat = coords.getAttribute("lat");
                                coords_lon = coords.getAttribute("lon");
                            } else {
                                street_title = result_list[i].getElementsByTagName('name')[0].textContent + '';
                            }

                            /*check if the item starts with the same letters as the text field value:*/

                            /*create a DIV element for each matching element:*/
                            let b = document.createElement("DIV");
                            /*make the matching letters bold:*/
                            if (sel_type == 'city') {
                                b.innerHTML = city_title + ' ' + '(' + region + ')';
                                b.innerHTML += "<input type='hidden' value='" + city_title + "' data-lat='" + coords_lat + "'  data-lon='" + coords_lon + "' data-selectedCityCode='" + city_code + "'>";
                            } else {
                                b.innerHTML = street_title;
                                b.innerHTML += "<input type='hidden' value='" + street_title + "'>";
                            }
                            /*insert a input field that will hold the current array item's value:*/

                            /*execute a function when someone clicks on the item value (DIV element):*/
                            b.addEventListener("click", function (e) {
                                /*insert the value for the autocomplete text field:*/
                                inp.value = this.getElementsByTagName("input")[0].value;

                                if (sel_type == 'city') {

                                    if (this.getElementsByTagName("input")[0].getAttribute("data-lat").length > 0 && this.getElementsByTagName("input")[0].getAttribute("data-lon").length > 0) {
                                        map.setView([this.getElementsByTagName("input")[0].getAttribute("data-lat"), this.getElementsByTagName("input")[0].getAttribute("data-lon")], settings.mapSearchZoom)
                                    }

                                    document.getElementById('searchCity').setAttribute('data-selectedCityTitle', this.getElementsByTagName("input")[0].value);
                                    document.getElementById('searchCity').setAttribute('data-selectedCityCode', this.getElementsByTagName("input")[0].getAttribute("data-selectedCityCode"));
                                }

                                /*close the list of autocompleted values,
                                (or any other open lists of autocompleted values:*/
                                closeAllLists();
                            });
                            a.appendChild(b);


                        }
                    }


                },
                error: function (jqxhr, status, errorMsg) {
                    console.log(status + ": " + errorMsg);
                }
            });


        });
        /*execute a function presses a key on the keyboard:*/
        inp.addEventListener("keydown", function (e) {

            var x = document.getElementById(this.id + "autocomplete-list");
            if (x) x = x.getElementsByTagName("div");
            if (e.keyCode == 40) {
                /*If the arrow DOWN key is pressed,
                increase the currentFocus variable:*/
                currentFocus++;
                /*and and make the current item more visible:*/
                addActive(x);
            } else if (e.keyCode == 38) { //up
                /*If the arrow UP key is pressed,
                decrease the currentFocus variable:*/
                currentFocus--;
                /*and and make the current item more visible:*/
                addActive(x);
            } else if (e.keyCode == 13) {
                /*If the ENTER key is pressed, prevent the form from being submitted,*/
                e.preventDefault();
                if (currentFocus > -1) {
                    /*and simulate a click on the "active" item:*/
                    if (x) x[currentFocus].click();
                }
            }
        });

        function addActive(x) {
            /*a function to classify an item as "active":*/
            if (!x) return false;
            /*start by removing the "active" class on all items:*/
            removeActive(x);
            if (currentFocus >= x.length) currentFocus = 0;
            if (currentFocus < 0) currentFocus = (x.length - 1);
            /*add class "autocomplete-active":*/
            x[currentFocus].classList.add("autocomplete-active");
        }

        function removeActive(x) {
            /*a function to remove the "active" class from all autocomplete items:*/
            for (var i = 0; i < x.length; i++) {
                x[i].classList.remove("autocomplete-active");
            }
        }

        function closeAllLists(elmnt) {
            /*close all autocomplete lists in the document,
            except the one passed as an argument:*/
            var x = document.getElementsByClassName("autocomplete-items");
            for (var i = 0; i < x.length; i++) {
                if (elmnt != x[i] && elmnt != inp) {
                    x[i].parentNode.removeChild(x[i]);
                }
            }
        }

        /*execute a function when someone clicks in the document:*/
        document.addEventListener("click", function (e) {
            closeAllLists(e.target);
        });
    };
    measoftMap.showMap = function () {
        var xml = '<?xml version="1.0" encoding="UTF-8" ?><pvzlist><auth extra="' + settings.client_id + '"></auth>';
        if (settings.townBlock) {
            measoftMap.town = document.getElementById(settings.townBlock).value;
			if(settings.townRegexp){
			measoftMap.town=measoftMap.town.replace( settings.townRegexp, "" );
			}			
        }
        if (settings.client_code) {
            xml = xml + '<client_code>' + settings.client_code + '</client_code>';
        }

        for (var fname in settings.filter) {

            let settings_filter_value = settings.filter[fname].toString().replace(",", ".");

            if ((fname == 'store' && parseInt(settings_filter_value) > 0) || (fname == 'maxweight' && parseFloat(settings_filter_value) > 0) || (settings_filter_value == 'YES')) {
                if(fname=='store') fname='parentcode';
                xml = xml + '<' + fname + '>' + settings_filter_value + '</' + fname + '>';
            }
        }

        if (measoftMap.town) {
            xml = xml + '<town>' + measoftMap.town + '</town>';
        }

        xml = xml + '<json>YES</json></pvzlist>';

        ajax(api_url, {
            data: xml,
            success: function (response) {
                if (response !== undefined) {
                    //console.log(response);
                    var pvz_list = JSON.parse(response);
                    marksData = [];
                    for (var i = 0, l = pvz_list.length; i < l; i++) {
                        var la = pvz_list[i].la;
                        var lo = pvz_list[i].lo;
                        if (pvz_list[i].co != '' && pvz_list[i].na != '' && (la != '' && lo != '')) {
                            marksData.push({
                                'coordinates': [la, lo],
                                'code': pvz_list[i].co,
                                'name': pvz_list[i].na
                            });
                        }
                    }

                    if (marksData.length > 0) {
                        initOsm(marksData);
                    } else {
                        initOsm(marksData);
                        measoftMap.showError('Пунктов самовывоза не найдено');
                    }
                } else {					
                    measoftMap.showError('Произошла ошибка',1);
                }
            },
            error: function (jqxhr, status, errorMsg) {
                console.log(status + ": " + errorMsg);
            }
        });
    };

    function initOsm(markData) {
        if (map) {
            var currentMapPoss = map.getBounds();
            if (placeCoords != 0) {
                currentMapPoss = placeCoords;
            } else {
                currentMapPoss = map.getBounds();
            }
            map.remove();
        }

        mapReady = false;
        measoftMap.objects = [];

        map = L.map('map_content_block', {
            center: settings.centerCoords,
            minZoom: 2,
            zoom: 4
        });

        measoftMap.map = map;
        map.whenReady(function () {
            mapReady = true;
        });

        // добавим для соблюдения условий лицензии
//		L.tileLayer( 'http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
        L.tileLayer('http://{s}.osm.courierexe.ru/tiles.php?z={z}&x={x}&y={y}.png', {
            attribution: '&copy; <a href="https://courierexe.ru/">MeaSoft</a>&nbsp;&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a>',
            subdomains: ['a', 'b', 'c']
        }).addTo(map);

        var markerClusters = L.markerClusterGroup({
            showCoverageOnHover: false
        });

        var bounds = new L.LatLngBounds();
        for (var i = 0; i < markData.length; ++i) {
            var cur = markData[i];
            var data = measoftMap.createListItem(markData[i], i);

            measoftMap.objects.push(data);

            var m = L.marker(cur.coordinates);

            m.number = i;

            m.bindPopup(data.balloonContentHeader);

            m.on("click", function (e) {
                var marker = e.target;
                setMarkerContent(marker);
            });

            markerClusters.addLayer(m);
            bounds.extend(m.getLatLng());
        }


        measoftMap.showMapBlock();
        //TODO
        if (placeCoords != 0) {
			
            map.addLayer(markerClusters);
            map.setView(currentMapPoss, settings.mapZoom);
            placeCoords = 0;
        } else {
            if (settings.townBlock) {
                if (markData.length) {
                    map.fitBounds(bounds);
                }
                map.addLayer(markerClusters);
                if (markData.length) {
                    map.setView(bounds.getCenter(), settings.mapZoom);
                }
                
            } else {				
                map.addLayer(markerClusters);
				if (typeof currentMapPoss !== 'undefined') {
				map.fitBounds(currentMapPoss);              
				}				
            }


        }
    }

    measoftMap.createListItem = function (item) {
        return {
            hintContent: item.name,
            balloonContentHeader: item.name,
            balloonContentBody: '',
            code: item.code,
            name: item.name,
            phone: '',
            address: '',
            worktime: '',
            maxweight: '',
			parentcode: ''
        };
    };

    measoftMap.selectPVZ = function (number) {
        let object = measoftMap.objects[number],
            name = object.name,
            code = object.code,
            address = object.address,
            event1 = null,
            event2 = null,
            event3 = null;

        if (settings.pvzCodeSelector.length > 0 && document.querySelector(settings.pvzCodeSelector) !== null) {
            document.querySelector(settings.pvzCodeSelector).value = code;
            event1 = document.createEvent('Event');
            event1.initEvent('change', true, true);
            document.querySelector(settings.pvzCodeSelector).dispatchEvent(event1);

        }
        if (settings.pvzNameSelector.length > 0 && document.querySelector(settings.pvzNameSelector) !== null) {
            document.querySelector(settings.pvzNameSelector).value = name + "; " + address;
            event2 = document.createEvent('Event');
            event2.initEvent('change', true, true);
            document.querySelector(settings.pvzNameSelector).dispatchEvent(event2);
        }
        if (settings.pvzAddressSelector.length > 0 && document.querySelector(settings.pvzAddressSelector) !== null) {
            document.querySelector(settings.pvzAddressSelector).value = address;
            event3 = document.createEvent('Event');
            event3.initEvent('change', true, true);
            document.querySelector(settings.pvzAddressSelector).dispatchEvent(event3);
        }

		measoftMap.savePvzData(number);

        measoftMap.map.closePopup();
        measoftMap.close();
    };

    function setMarkerContent(marker) {
        var m_num = marker.number;
        var code = measoftMap.objects[m_num].code;
        let pvz_button = "<div style=\"float:right;\"><input type=\"button\"  onClick=\"measoftMap.selectPVZ(" + m_num + ");\" value=\"Выбрать ПВЗ\" /></div>";
        var xml = '<?xml version="1.0" encoding="UTF-8" ?><pvzlist><auth extra="' + settings.client_id + '"></auth><code>' + code + '</code></pvzlist>';
        if (measoftMap.objects[m_num].address == '') {

            marker.setPopupContent('Загрузка...');

            ajax(api_url, {
                data: xml,
                success: function (response) {

                    if (window.DOMParser) {
                        var parser = new DOMParser();
                        var xmlDoc = parser.parseFromString(response, "text/xml");
                    } else // Internet Explorer
                    {
                        var xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
                        xmlDoc.async = false;
                        xmlDoc.loadXML(response);
                    }

                    var pvz_list = xmlDoc.getElementsByTagName('pvz');

                    if (pvz_list.length > 0) {
                        measoftMap.objects[m_num].balloonContentBody = 'Адрес: ' + pvz_list[0].getElementsByTagName('address')[0].innerHTML + '<br>Телефон: ' + pvz_list[0].getElementsByTagName('phone')[0].innerHTML + '<br>Часы работы: ' + pvz_list[0].getElementsByTagName('worktime')[0].innerHTML + '<br>Максимальный вес: ' + pvz_list[0].getElementsByTagName('maxweight')[0].innerHTML + '<hr>' + pvz_list[0].getElementsByTagName('comment')[0].innerHTML;

                        measoftMap.objects[m_num].address = pvz_list[0].getElementsByTagName('address')[0].innerHTML;
                        measoftMap.objects[m_num].phone = pvz_list[0].getElementsByTagName('phone')[0].innerHTML;
                        measoftMap.objects[m_num].worktime = pvz_list[0].getElementsByTagName('worktime')[0].innerHTML;
                        measoftMap.objects[m_num].maxweight = pvz_list[0].getElementsByTagName('maxweight')[0].innerHTML;
						measoftMap.objects[m_num].parentcode = pvz_list[0].getElementsByTagName('parentcode')[0].innerHTML;
						

                        marker.setPopupContent('<div style="float:left;"><strong>' + measoftMap.objects[m_num].balloonContentHeader + '</strong></div>' + pvz_button + '<div style="clear:both"></div><hr/>' + measoftMap.objects[m_num].balloonContentBody);

                      //  measoftMap.savePvzData(m_num);
                    } else {
                        measoftMap.objects[m_num].balloonContentBody = 'Не удалось получить информацию о выбранном объекте.';
                    }
                },
                error: function (jqxhr, status, errorMsg) {
                    console.log(status + ": " + errorMsg);
                }
            });
        }

//		measoftMap.savePvzData(m_num);
    }

    measoftMap.choicePvz = function (placemark) {
        var xml = '<?xml version="1.0" encoding="UTF-8" ?><pvzlist><auth extra="' + settings.client_id + '"></auth><code>' + placemark.properties.get('code') + '</code></pvzlist>';

        var pm_num = 77;//placemark.properties.get('number');

        if (measoftMap.objects[pm_num].properties.get('address') == '') {

            measoftMap.objects[pm_num].properties.set('balloonContentBody', 'Загрузка...');

            ajax(api_url, {
                data: xml,
                success: function (response) {
                    if (window.DOMParser) {
                        var parser = new DOMParser();
                        var xmlDoc = parser.parseFromString(response, "text/xml");
                    } else // Internet Explorer
                    {
                        var xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
                        xmlDoc.async = false;
                        xmlDoc.loadXML(response);
                    }

                    var pvz_list = xmlDoc.getElementsByTagName('pvz');

                    if (pvz_list.length > 0) {
                        measoftMap.objects[pm_num].properties.set('balloonContentBody', 'Адрес: ' + pvz_list[0].getElementsByTagName('address')[0].innerHTML + '<br>Телефон: ' + pvz_list[0].getElementsByTagName('phone')[0].innerHTML + '<br>Часы работы: ' + pvz_list[0].getElementsByTagName('worktime')[0].innerHTML + '<br>Максимальный вес: ' + pvz_list[0].getElementsByTagName('maxweight')[0].innerHTML + '<hr>' + pvz_list[0].getElementsByTagName('comment')[0].innerHTML);

                        measoftMap.objects[pm_num].properties.set('address', pvz_list[0].getElementsByTagName('address')[0].innerHTML);
                        measoftMap.objects[pm_num].properties.set('phone', pvz_list[0].getElementsByTagName('phone')[0].innerHTML);
                        measoftMap.objects[pm_num].properties.set('worktime', pvz_list[0].getElementsByTagName('worktime')[0].innerHTML);
                        measoftMap.objects[pm_num].properties.set('maxweight', pvz_list[0].getElementsByTagName('maxweight')[0].innerHTML);

                        measoftMap.savePvzData(pm_num);
                    } else {
                        measoftMap.objects[pm_num].properties.set('balloonContentBody', 'Не удалось получить информацию о выбранном объекте.');
                    }
                },
                error: function (jqxhr, status, errorMsg) {
                    console.log(status + ": " + errorMsg);
                }
            });
        }

        measoftMap.savePvzData(pm_num);
    }

    measoftMap.savePvzData = function (object_number) {
        // Записываем код и адрес выбранного ПВЗ
        document.getElementById('pvz_code').value = measoftMap.objects[object_number].code;
        document.getElementById('pvz_address').value = measoftMap.objects[object_number].address;
        document.getElementById('pvz_phone').value = measoftMap.objects[object_number].phone;
        document.getElementById('pvz_worktime').value = measoftMap.objects[object_number].worktime;
        document.getElementById('pvz_maxweight').value = measoftMap.objects[object_number].maxweight;
		document.getElementById('pvz_parentcode').value = measoftMap.objects[object_number].parentcode;
		
        if (settings.choicePvzCallback) {
            settings.choicePvzCallback(measoftMap.objects[object_number].code);
        }
    };

    function getMapWindowHtml() {
        var window_style = 'background-color: #fff; padding: 0; border: 1px solid #ccc; box-shadow: 0 0 10px rgba(0, 0, 0, 0.3); ';

        if (settings.mapSize.width == 'auto') {
            window_style = window_style + 'width: 100%;';
        } else {
            if (settings.mapSize.width > window.innerWidth) {
                settings.mapSize.width = parseInt(window.innerWidth - (window.innerWidth / 100 * 10));
            }
            window_style = window_style + 'width: ' + settings.mapSize.width + 'px;';
        }

        if (settings.mapSize.height == 'auto') {
            window_style = window_style + 'height: 100%;';
        } else {
            window_style = window_style + 'height: ' + settings.mapSize.height + 'px;';
        }

        if (window.innerWidth <= 567) {
            window_style = window_style + 'max-height:500px;min-height:unset;';
        }

        if ((settings.showMapButton == '1') || (settings.showMapButton == '2')) {
            var html = '';
            if (settings.windowFixedPosition == '1') {

                document.addEventListener("click", function (e) {
                    e = e || window.event;
                    let target = e.target || e.srcElement,
                        targetEl = target.getAttribute('id');
                    if (targetEl == 'measoftmap_overlay') {
                        measoftMap.close();
                    }
                });

                html = html + '<div id="measoftmap_overlay" style="z-index: 1000000; background:rgba(0, 0, 0, 0.3); top:0; right:0; bottom:0; left:0; position:fixed; display: none;"></div>';
                window_style = window_style + 'position: fixed; margin: 2% 5%; z-index: 11000000; right: 0px; top: 0px; bottom: 0px; left: 0px; width: 90%; height: 90%;';
            } else {
                window_style = window_style + 'position: absolute; top: 100; margin-top: 10px;';
            }

            if (settings.showMapButton == '1') {
                html += '<input type="button" class="btn btn-default" id="' + settings.buttonBlock + '" label="Выбрать пункт самовывоза из списка" onClick="measoftMap.open()" value="' + settings.showMapButtonCaption + '">';
            }
            return html + '<div id="pvz_map_container" style="' + window_style + ' display: none;z-index:1000001;">' + measoftMap.showMapWindowBlock() + '</div>';
        } else {
            return '<div id="pvz_map_container" style="' + window_style + '">' + measoftMap.showMapWindowBlock() + '</div>';
        }
    }

    measoftMap.showMapWindowBlock = function () {
        var mwhtml = '';

        if ((settings.showMapButton == '1') || (settings.showMapButton == '2')) {
            mwhtml = mwhtml + '<a href="javascript:measoftMap.close()" style="float: right; padding: 8px 12px;">Закрыть</a>';
        }

        // Подключаем форму поиска
        //mwhtml += measoftMap.searchForm();

        mwhtml = mwhtml + '<div id="map_filter_block" style="padding: 8px; margin-bottom: 5px;">' + measoftMap.showMapFilter() + '</div>' + measoftMap.showSearchAddress() + '\
			<div id="map_content_block" style="display: none; position: relative; bottom: 0; width: 100%; height: ' + (settings.mapSize.height == 'auto' ? '100%' : (settings.mapSize.height - (settings.allowedFilterParams.length > 0 ? 90 : (90 - 19.5))) + 'px') + ';' + (settings.mapSize.height == 'auto' ? 'min-height:' + window.innerHeight + 'px;' : '') + (window.innerWidth <= 567 ? 'max-height:500px;min-height:unset;' + (settings.mapSize.height == 'auto' ? 'height:500px;' : '') : '') + '">\
			<div id="measoft_map" style="width: 100%; height: 100%; position: absolute;"></div></div>\
			<div id="map_load_block" style="display: ;"><p style="text-align: center;">Загрузка карты...</p></div>\
			<div id="map_error_block" style="display: none;"><p id="error_text" style="text-align: center;">Пунктов самовывоза не найдено</p></div>\
			<!-- id выбранной точки и адрес ПВЗ -->\
			<input type="hidden" value="0" name="pvz_code" id="pvz_code" />\
			<input type="hidden" value="" name="pvz_address" id="pvz_address" />\
			<input type="hidden" value="" name="pvz_phone" id="pvz_phone" />\
			<input type="hidden" value="" name="pvz_worktime" id="pvz_worktime" />\
			<input type="hidden" value="" name="pvz_parentcode" id="pvz_parentcode" />\
			<input type="hidden" value="" name="pvz_maxweight" id="pvz_maxweight" />';

        return mwhtml;
    };


    measoftMap.showSearchAddress = function () {

        if (settings.showSearchAddress == '1') {

            let html = '<style>#map_filter_address_block #searchAddressButton{text-align: center !important; padding: 5px 10px 5px 9px;} #map_filter_address_block input {max-width: 140px !important;width:20%; padding: 5px;margin-right: 5px;border: 1px solid gray;}#map_filter_address_block #searchStreetautocomplete-list{left: 171px;}#map_filter_address_block #searchCityautocomplete-list{left: 8px;}#map_filter_address_block .autocomplete {  position: relative;  display: inline-block;  width: 300px;}#map_filter_address_block .autocomplete-items {width:300px;  position: absolute;  border: 1px solid #d4d4d4;  border-bottom: none;  border-top: none;  z-index: 99999;  left: 0;  right: 0;}#map_filter_address_block .autocomplete-items div {  padding: 10px;  cursor: pointer;  background-color: #fff;  border-bottom: 1px solid #d4d4d4;}#map_filter_address_block .autocomplete-items div:hover {  background-color:#e9e9e9;}#map_filter_address_block .autocomplete-active {  background-color: DodgerBlue !important;  color: #ffffff;} #map_filter_address_block > label {  display: inline-block; margin-right: 8px; margin-left: 2px; margin-bottom: 0px; font-size:100%; font-weight:100;} .leaflet-popup-content {max-height: 300px;overflow: auto;} #map_filter_block{display: inline-flex;flex-wrap: wrap;} #map_filter_block div {margin-right: 10px;} #measoftMapBlock input[type="checkbox"]{vertical-align:text-bottom;} </style>\
			<div id="map_filter_address_block" style="padding: 8px; margin-bottom: 5px;positon: relative;">\
<input data-selectedCityCode="" data-selectedCityTitle="" placeholder="город" id="searchCity" />\
<input placeholder="улица" id="searchStreet"  />\
<input placeholder="дом" id="searchHouse" />\
<input type="button" id="searchAddressButton" value="Поиск" onClick="measoftMap.MoveToAddress();" />\
<div id="searchResults" style="position: absolute; z-index: 99999; background-color: #fff; margin-top: 5px;"></div>\
			</div>\
			';


            return html;
        }

        return '';
    };

    measoftMap.showMapFilter = function () {
        if (settings.allowedFilterParams.length > 0) {
            var html = '<style>#map_filter_block > label { display: inline-block; margin-right: 8px; margin-left: 2px; margin-bottom: 0px; }</style>';

            for (let i = 0; i < settings.allowedFilterParams.length; i++) {
                if (filterCaptions[settings.allowedFilterParams[i]] !== undefined && filterTypes[settings.allowedFilterParams[i]] !== undefined && filterTypes[settings.allowedFilterParams[i]]=='checked') {
                   if (settings.filter[settings.allowedFilterParams[i]] == 'YES') {
                       var checked = 'checked';
                       var new_val = 'NO';
                   } else {
                       var checked = '';
                       var new_val = 'YES';
                   }
                }

                if (filterCaptions[settings.allowedFilterParams[i]] !== undefined) {
                    var fcaption = filterCaptions[settings.allowedFilterParams[i]];
                    if (filterCaptions[settings.allowedFilterParams[i]] !== undefined && filterTypes[settings.allowedFilterParams[i]] !== undefined && filterTypes[settings.allowedFilterParams[i]]=='checked') {
                        html = html + '<div><input type="checkbox" id="cb_' + settings.allowedFilterParams[i] + '" id="' + settings.allowedFilterParams[i] + '" ' + checked + ' onclick="measoftMap.applyFilter(\'' + settings.allowedFilterParams[i] + '\', \'' + new_val + '\')"><label for="cb_' + settings.allowedFilterParams[i] + '">' + fcaption + '</label></div>';
                    }else if (filterCaptions[settings.allowedFilterParams[i]] !== undefined && filterTypes[settings.allowedFilterParams[i]] !== undefined && filterTypes[settings.allowedFilterParams[i]]=='select') {
                        html = html + '<div><select id="cb_' + settings.allowedFilterParams[i] + '" id="' + settings.allowedFilterParams[i] + '" ' +  ' onchange="measoftMap.applyFilter(\'' + settings.allowedFilterParams[i] + '\', \'select\')"><option value="">Выберите компанию</option></select></div>';
                    }

                }
            }

            return html;

        }

        return '';


    };

    measoftMap.applyFilter = function (param, value) {
        if(value=='select'){
            let e = document.getElementById('cb_' + param);
            let selStore = e.options[e.selectedIndex].value;
            settings.filter[param] = selStore;
        }else{
            var filter_new_val = 'YES';
            if (value == 'YES') {
                settings.filter[param] = value;
                filter_new_val = 'NO';
            } else {
                settings.filter[param] = '';
            }
            document.getElementById('cb_' + param).setAttribute("onclick", "measoftMap.applyFilter('" + param + "', '" + filter_new_val + "')");
        }

        measoftMap.showMap();
    };

    measoftMap.showMapBlock = function () {
        document.getElementById('map_error_block').style.display = 'none';
        document.getElementById('map_load_block').style.display = 'none';
        document.getElementById('map_content_block').style.display = '';
    };

    measoftMap.hideMapBlock = function () {
        document.getElementById('map_content_block').style.display = 'none';

        document.getElementById('map_error_block').style.display = 'none';
        document.getElementById('map_load_block').style.display = '';
    };

    measoftMap.showError = function (error_text, hide) {

        if (hide == 1) {
            document.getElementById('map_load_block').style.display = 'none';
            document.getElementById('map_content_block').style.display = 'none';

            document.getElementById('error_text').innerHTML = error_text;
            document.getElementById('map_error_block').style.display = '';
        } else {
            if (typeof measoftMap.map !== 'undefined') {
                if (typeof measoftMap.map.popups !== 'undefined') {

                    measoftMap.map.removePopup(measoftMap.map.popups[0]);
                }
                var MeasoftPopup = L.popup()
                    .setLatLng([measoftMap.map.getCenter().lat, measoftMap.map.getCenter().lng])
                    .setContent(error_text)
                    .openOn(measoftMap.map);
                measoftMap.map.setView([measoftMap.map.getCenter().lat, measoftMap.map.getCenter().lng], 2);
            }
        }

    };

    measoftMap.getSelectedPvzData = function () {
        return {
            'code': document.getElementById('pvz_code').value,
            'towncode': document.getElementById('pvz_towncode').value,
            'address': document.getElementById('pvz_address').value,
            'phone': document.getElementById('pvz_phone').value,
            'worktime': document.getElementById('pvz_worktime').value,
            'maxweight': document.getElementById('pvz_maxweight').value,
        };
    };

    measoftMap.open = function () {
        document.getElementById('pvz_map_container').style.display = '';
        document.getElementById(settings.buttonBlock).setAttribute('onclick', 'measoftMap.close()');

        if (settings.windowFixedPosition == '1') {
            document.getElementById('measoftmap_overlay').style.display = '';
        }

        validateSize();
        measoftMap.isOpened = true;

        if (settings.townBlock && (measoftMap.town != document.getElementById(settings.townBlock).value)) {
            measoftMap.showMap();
        }
    };

    function validateSize() {
        if (!mapReady) {
            window.setTimeout(validateSize, 100);
            return;
        }
        measoftMap.map.invalidateSize();
    }

    measoftMap.close = function () {
        document.getElementById('pvz_map_container').style.display = 'none';
        document.getElementById(settings.buttonBlock).setAttribute('onclick', 'measoftMap.open()');

        if (settings.windowFixedPosition == '1') {
            document.getElementById('measoftmap_overlay').style.display = 'none';
        }

        measoftMap.isOpened = false;
    };

    measoftMap.clear = function () {
        document.getElementById('pvz_name').value = '';

        // Очищаем служебные поля
        document.getElementById('pvz_code').value = '';
        document.getElementById('pvz_address').value = '';
        document.getElementById('pvz_phone').value = '';
        document.getElementById('pvz_worktime').value = '';
        document.getElementById('pvz_maxweight').value = '';

        //Снимаем выделение с ПВЗ в списке
        //document.getElementById('pvz_list option:selected').selected = false;

        closePvzMap();
    };

    // Разметка формы поиска
    measoftMap.searchForm = function () {
        let html = '<div class="wrap" style="width: 70%;">\
				<div class="search" style="width: 100%; position: relative; display: flex;">\
				<input type="text" id="searchTerm" placeholder="Введите наименование города" onKeyDown="measoftMap.placeSearcher()" style="width: 100%; border: 3px solid #BBB; border-right: none;\
					padding: 5px; border-radius: 5px 0 0 5px; outline: none; color: #000;">\
				<button type="submit" class="searchButton" style="width: 70px; height: 36px; border: 1px solid #bbb; background: #bbbbbb; text-align: center;\
					color:#fff; border-radius: 0 5px 5px 0; cursor: pointer; font-size: 20px;" onKeyDown="measoftMap.placeSearcher()">Поиск\
				</button>\
				</div>\
				<div id="searchResult" style="position: absolute; z-index: 99999; background-color: #fff; margin-top: 25px;"></div>\
				</div>';
        return html;
    }

    let searchResultIndex = -1;
    window.addEventListener("keydown", function (event) {
        if (event.defaultPrevented) {
            return;
        }

        let searchObject = document.getElementById('searchResult');
        if (searchObject) {
            let max = searchObject.childNodes.length;
            var handled = false;
            if (event.keyCode == 40) {
                if (searchResultIndex <= 0) {
                    searchResultIndex = 0;
                }
                if (searchResultIndex < max) {
                    document.getElementById("st" + searchResultIndex).focus();
                    searchResultIndex++;
                } else {
                    searchResultIndex = max;
                }
            } else if (event.keyCode == 38) {
                if (searchResultIndex <= 0) {
                    searchResultIndex = 0;
                }
                if (searchResultIndex < 6) {
                    searchResultIndex--;
                    document.getElementById("st" + searchResultIndex).focus();
                }
            }

            if (event.keyCode == 32) {
                console.log(event.target.attributes[4].textContent + '' + event.target.attributes[5].textContent);
                measoftMap.placeSearcher(event.target.attributes[4].textContent + '', event.target.attributes[5].textContent)
            }
            if (handled) {
                event.preventDefault();
            }
        }
    }, true);

    // Поиск населённых пунктов или ПВЗ
    measoftMap.placeSearcher = function (placeName, placeFullName) {
        let obj = '';
        let xml = '';

        if (placeFullName > 0) {
            document.getElementById("searchTerm").value = placeName;
            document.getElementById('searchResult').innerHTML = '';
            xml = '<?xml version="1.0" encoding="UTF-8" ?><pvzlist><auth extra="8"></auth><town>' + parseInt(placeFullName) + '</town></pvzlist>';
        } else if (placeName.length <= 2) {
            let name = document.getElementById("searchTerm").value;
            xml = '<?xml version="1.0" encoding="UTF-8" ?><townlist><conditions><namecontains>' + name + '</namecontains></conditions><limit><limitcount>5</limitcount></limit></townlist>';
        }

        ajax(api_url, {
            data: xml,
            success: function (response) {

                if (window.DOMParser) {
                    var parser = new DOMParser();
                    var xmlDoc = parser.parseFromString(response, "text/xml");
                } else // Internet Explorer
                {
                    var xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
                    xmlDoc.async = false;
                    xmlDoc.loadXML(response);
                }

                if (placeFullName > 0) {
                    var place_list = xmlDoc.getElementsByTagName('pvz');
                } else if (placeName.length <= 2) {
                    var place_list = xmlDoc.getElementsByTagName('town');
                }

                obj = place_list;

                if (placeFullName > 0) {
                    measoftMap.moveToPlace(obj);
                } else if (placeName.length <= 2) {
                    measoftMap.showSearcherResult(obj);
                }

                if (place_list.length == 0) {
                    measoftMap.moveToPlace(obj);
                }
            },
            error: function (jqxhr, status, errorMsg) {
                console.log(status + ": " + errorMsg);
            }
        });
    }

    // Поиск населённых пунктов или ПВЗ
    measoftMap.coordsSearcher = function (placeName, placeFullName) {
        let obj = '';
        let xml = '';

        if (placeFullName > 0) {
            document.getElementById("searchTerm").value = placeName;
            document.getElementById('searchResult').innerHTML = '';
            xml = '<?xml version="1.0" encoding="UTF-8" ?><pvzlist><auth extra="8"></auth><town>' + parseInt(placeFullName) + '</town></pvzlist>';
        } else if (placeName.length <= 2) {
            let name = document.getElementById("searchTerm").value;
            xml = '<?xml version="1.0" encoding="UTF-8" ?><townlist><conditions><namecontains>' + name + '</namecontains></conditions><limit><limitcount>5</limitcount></limit></townlist>';
        }

        ajax(api_url, {
            data: xml,
            success: function (response) {

                if (window.DOMParser) {
                    var parser = new DOMParser();
                    var xmlDoc = parser.parseFromString(response, "text/xml");
                } else // Internet Explorer
                {
                    var xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
                    xmlDoc.async = false;
                    xmlDoc.loadXML(response);
                }

                if (placeFullName > 0) {
                    var place_list = xmlDoc.getElementsByTagName('pvz');
                } else if (placeName.length <= 2) {
                    var place_list = xmlDoc.getElementsByTagName('town');
                }

                obj = place_list;

                if (placeFullName > 0) {
                    measoftMap.moveToPlace(obj);
                } else if (placeName.length <= 2) {
                    measoftMap.showSearcherResult(obj);
                }

                if (place_list.length == 0) {
                    measoftMap.moveToPlace(obj);
                }
            },
            error: function (jqxhr, status, errorMsg) {
                console.log(status + ": " + errorMsg);
            }
        });
    }

    // Визуализация результата поиска
    measoftMap.showSearcherResult = function (obj) {
        let name = document.getElementById("searchTerm").value;
        let tag = document.getElementById('searchResult');
        // Показать форму если символов 2+
        if (name.length > 1) {
            tag.innerHTML = '';
            for (let i = 0; i < obj.length; i++) {
                var tmp = obj[i].getElementsByTagName('name')[1].textContent + '';
                let code = obj[i].getElementsByTagName('code')[0].textContent + '';
                let rigion = obj[i].getElementsByTagName('city')[0].getElementsByTagName('name')[0].textContent + '';
                tag.innerHTML += '<div id="st' + i + '" style="cursor:pointer;border-radius:5px;-webkit-border-radius:5px;-moz-border-radius:5px;-khtml-border-radius:5px; background-color: #fff;\
				-o-border-radius:5px;-icab-border-radius:5px;-ms-border-radius:5px; padding: 3px; border: 1px solid #bbb; margin-top: 1px; height: 20px; font-size: 16px;"\
				onClick="measoftMap.placeSearcher(\'' + tmp + '\', \'' + code + '\')" tabindex="0" tmp="' + tmp + '" code="' + code + '">' + tmp + ' (' + rigion + ')</div>';
            }
        } else {
            tag.innerHTML = '';
        }
    };

    // Изменение фокуса карты согласно выбранному городу
    measoftMap.moveToPlace = function (obj) {
        if (obj.length != 0) {
            for (let i = 0; i < obj.length; ++i) {
                let latitude = obj[i].getElementsByTagName('latitude')[0].innerHTML;
                let longitude = obj[i].getElementsByTagName('longitude')[0].innerHTML;
                if (latitude.length > 5 && longitude.length > 5) {
                    placeCoords = [latitude.toString(), longitude.toString()];
                    measoftMap.showMap();
                    break;
                }
            }
        } else {
            let tag = document.getElementById('searchResult');
            tag.innerHTML = '';
            tag.innerHTML += '<div style="cursor:pointer;border-radius:5px;-webkit-border-radius:5px;-moz-border-radius:5px;-khtml-border-radius:5px; background-color: #faa;\
				-o-border-radius:5px;-icab-border-radius:5px;-ms-border-radius:5px; padding: 3px; border: 1px solid #bbb; margin-top: 1px; height: 20px; font-size: 16px;">Данных нет</div>';
        }
    };


    measoftMap.preloadImages = function (array) {
        let images = [];
        for (var i = 0; i < array.length; i++) {
            images[i] = new Image();
            images[i].src = array[i];
        }
    }


    measoftMap.ClearMyselfAddress = function () {
        if (myselfLayer !== null) {
            myselfLayer.remove();
        }
    };

    measoftMap.ShowMyselfAddress = function (coords) {

        map.setView(coords, settings.mapSearchZoom);

        let greenIcon = new L.Icon({
            iconUrl: 'https://cdn.rawgit.com/pointhi/leaflet-color-markers/master/img/marker-icon-2x-green.png',
            shadowUrl: 'https://cdnjs.cloudflare.com/ajax/libs/leaflet/0.7.7/images/marker-shadow.png',
            iconSize: [25, 41],
            iconAnchor: [12, 41],
            popupAnchor: [1, -34],
            shadowSize: [41, 41]
        });

        myselfLayer = L.marker(coords, {icon: greenIcon}).bindTooltip('Вы тут', {
            permanent: true,
            direction: 'top',
            offset: [0, -40]
        }).addTo(map);

    };


    measoftMap.MoveToAddress = function () {

        measoftMap.ClearMyselfAddress();

        document.getElementById('searchResults').innerHTML = '';

        let city = document.getElementById('searchCity').value,
            street = document.getElementById('searchStreet').value,
            house = document.getElementById('searchHouse').value,
            geo_code_url = 'https://home.courierexe.ru/geocode?extracode='+settings.client_id+'&geocode=' + (city + ', ' + city + ', ' + street + ', ' + house);


        if (!city.length || !street.length || !house.length) return false;
        ajax(geo_code_url, {

            success: function (response) {


                if (window.DOMParser) {
                    var parser = new DOMParser();
                    var xmlDoc = parser.parseFromString(response, "text/xml");
                } else // Internet Explorer
                {
                    var xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
                    xmlDoc.async = false;
                    xmlDoc.loadXML(response);
                }


                let pos = xmlDoc.getElementsByTagName('pos');

                if (pos !== undefined && pos.length > 0) {

                    pos = pos[0].innerHTML;

                    let coords = pos.split(" ");
					if(coords[0]!=0 && coords[1]!=0){
					placeCoords = [coords[1], coords[0]];	
					 measoftMap.ShowMyselfAddress(placeCoords);
					}
                    

                   

                }


            },
            error: function (jqxhr, status, errorMsg) {
                console.log(status + ": " + errorMsg);
            },

        });


    };


    //Объявление глобальной переменной
    window.measoftMap = measoftMap;
})(window);